#Common RestApi methods
API_COMMON_DEVICE_OFF = "off"
API_COMMON_DEVICE_ON = "on"
API_COMMON_GET_STATE = "get_state/"
API_COMMON_SET_STATE = "set_state/"
API_COMMON_GET_DEVICE_INFO = "get_device_info/"

#Client connection timeout in seconds
API_CLIENT_CONNECTION_TIMEOUT = 5

#Common RestApi status values
API_RESPONSE_STATUS_OK = "ok"
API_RESPONSE_STATUS_FAIL = "false"
API_RESPONSE_STATUS_INVALID = "invalid_action_name"

#Request base params keys
REQUEST_CHANNEL_KEY = "channel"
REQUEST_STATE_KEY = "state"

#Logger lib name
LOGGER_NAME = "foxrestapiclient_log"