from . import fox_base_device
from . import fox_dim1s2_device
from . import fox_dimmable_device
from . import fox_led2s2_device
from . import fox_r1s1_device
from . import fox_r2s2_device
from . import fox_rgbw_device
from . import fox_str1s2_device